function divs = divide_interval( counts, separation )
% Used in computing visualisation of multi-dimensional contingency tables
% Divides the interval [0,1] into sub-intervals with sizes proportional to
% the elements of the parameter counts, and with separations determined by the parameter
% separation. 
% counts is a vector of at least one positive value
% separation is a real value in [0,1) that determines the amount of
%       separation between successive intervals
% divs is a 2 x length( counts ) matrix containing the starts (in the first row) and ends 
%  (in the second row) of the sub-intervals of [0,1] produced

if nargin == 1
    separation = 0; 
end

% basic integrity checks 
assert( separation >= 0 ); 
assert( not( isempty( counts )) ); 
assert( isnumeric( counts ) ); 
assert( all( counts >= 0 ) ); 
assert( sum( counts ) > 0 ); 

counts = counts(:)'; % ensures it is a row vector. 
nc = length( counts ) ; 

% trivial case -- separation has no effect, interval is not divided
if nc == 1
    divs = [0; 1]; 
    return; 
end

fracs =  counts / sum( counts ); 
sep = separation / (nc - 1); 

adjusted_fracs = (1 - separation ) * fracs;
adjusted_fracs( 1:end-1 ) = adjusted_fracs( 1:end-1 ) + sep; 

cum_adj_fracs = cumsum( adjusted_fracs ); 

interval_starts = [ 0 , cum_adj_fracs(1:end-1) ];
interval_ends = [ cum_adj_fracs(1:end-1) - sep, 1 ]; 

divs = [ interval_starts; interval_ends ];

end

