function patchspec = divide_patch( patchspec, level, datacol, thresholds, sep )
% patchspec = divide_patch( patchspec, level, datacol, thresholds, sep )
% arguments: 
%    patchspec is the root patch node
%    level is the level after division ( start with 1 )
%    datacol is a column linking horizontally to the study table
%    thresholds : optional parameter, vector of threshold values (default
%                           [0.5])
%    sep : optional parameter  specifying gap between divided patches
%            (default 0.15)
% a patchspec is a struct with the following fields:
%    xx, yy are 
%    divide_direction (1 for x, 2 for y )
%    idxx : vector of indices of members
%    children : a cell array of child patch-specs

if nargin < 5 
    sep = 0.15; % seems a reasonable default -- maybe 0.2 would be better? 
end
if nargin == 3 
    thresholds = 0.5; % default that works for logical vectors
end
if isempty( thresholds)
    error( 'Must supply vector of thresholds or use default of 0.5 ' );
end
if nargin < 3 
    error( 'Not enough arguments for divide_patch' ); 
end
if level < 1
    error( 'Level must be >= 1' ); 
else if level > 1
    nc = length( patchspec.children ); 
    for cx = 1:nc
        patchspec.children{cx} = divide_patch( patchspec.children{cx}, level - 1, datacol, thresholds, sep );
    end
else    
    %keyboard

    idxx = patchspec.idxx; % for brevity
    selected_data = datacol( idxx ); 

    [ counts data_subsetxx ] = bin_by_thresholds( selected_data, thresholds ); 
% data_subsetxx is a cell array of vectors of indices within selected_data
%  these indices must be reset to corresponding subsets of idxx before
%  being assigned into the children
    
    if ( sum( counts ) > 0 ) 
        unscaled_intervals = divide_interval( counts, sep );
    else % no data in this patch -- divide interval arbitrarily
        unscaled_intervals = [ zeros( 1, length( counts ) ) ; ones( 1, length( counts ) ) ] ; 
    end
    
    if patchspec.divide_direction == 1
        scaled_intervals = patchspec.xx(1) + (patchspec.xx(2) - patchspec.xx(1)) * unscaled_intervals;
    else
        scaled_intervals = patchspec.yy(1) + (patchspec.yy(2) - patchspec.yy(1)) * unscaled_intervals;
    end

    n_thresholds = length( thresholds ); 
    n_children = n_thresholds + 1; 
    children = cell( n_children, 1 ); 
    child_divide_direction = 3 - patchspec.divide_direction; % alternates between 1 and 2

    % create the children 
    for childx = 1:n_children
        children{childx}.idxx = idxx( data_subsetxx{ childx }  ); % these are indices relative to datacol
        children{childx}.divide_direction = child_divide_direction; 
        children{childx}.children = {};
        if patchspec.divide_direction == 1
            children{childx}.xx = scaled_intervals(:,childx); 
            children{childx}.yy = patchspec.yy; 
        else
            children{childx}.xx = patchspec.xx; 
            children{childx}.yy = scaled_intervals(:,childx); 
        end
    end

    patchspec.children = children; 
    end
end

end

