function neighbour_plot( tspts, zscores, low_neighbour_index, high_neighbour_index,...
    high_alpha, low_alpha, cmap ) 
% This plots a neighbour plot in a new figure, and turns off the axes...
% Called as: 
%   neighbour_plot( tspts, zscores, low_neighbour_index, high_neighbour_index,...
%       high_alpha, low_alpha, cmap )
% Only the first two arguments are absolutely required: all the others have
% default values, and need not be given. 
% Arguments:
%   tspts : Nx2 matrix givine the tsne (or pca, or whatever) coordinates of
%       the visualisation points
%   zscores : NxK matrix of the zscores (or whatever the coordinates are in
%       the high dimensional space )
%   low_neighbour_index : (default 1 ) a positive integer; this is the index of the
%       first neighbour of each point to use (so 1 means use the
%       nearest neighbour, 2 means second-nearest neighbour etc )
%   high_neighbour_index : (default 2 ) a positive integer >= low_neighbour_index
%       Typically 2 or 3 or 10 or so. 
%   high_alpha : a real number between 0 and 1 (default 0.5 ) This
%       specifies the transparency of the thick end of each arrow (lower
%       numbers mean more transparent, 1.0 means completely opaque)
%   low_alpha : a real number between 0 and 1 (default 0.3) This specifies
%       the transparency of the thin end of each arrow 
%   cmap : this is a 64x3 matrix of colours (can be one of the system
%       colormaps: jet is the default, but type help graph3d to see a range of useful colormaps )
%
% Result : no result is returned. 

if nargin < 3
    low_neighbour_index = 1; 
end
if nargin < 4
    high_neighbour_index = 2; 
end
if nargin < 5 
    high_alpha = 0.5;
end
if nargin < 6
    low_alpha = 0.3; 
end
if nargin < 7 
    cmap = jet; 
end
    
assert( high_neighbour_index >= low_neighbour_index, ...
    'High neighbour index must be greater than or equal to low neighbour index'); 

assert( size( cmap, 1 ) == 64 , 'Color map must have 64 rows');
assert( size( cmap, 2 ) == 3, ' Color map must have 3 columns'); 
assert( size( tspts, 1) == size( zscores, 1 ), 'There must be the same number of tspts and zscores' ); 
assert( size( tspts, 2) == 2, 'There must be exactly two columns in tspts' ); 

[neighbours neighbour_distances] = ...
    all_neighbours( euclidean_squared_distances( zscores ), ...
                    high_neighbour_index ); 

width = 0.02; 
mitre = 0.75; 
taper = 0.2;
neighbour_range = low_neighbour_index:high_neighbour_index; 
[ vertices faces vertex_colourx vertex_alphax] = ...
    make_vertices_faces_from_neighbours( ...
    tspts, neighbours(:,neighbour_range), ...
    neighbour_distances(:,neighbour_range), width, mitre, taper,...
    'descend',1.0, high_alpha, low_alpha );

%figure
colormap( cmap );
p = patch('Faces',faces,'Vertices',vertices,'FaceColor','interp',...
    'FaceVertexCData', vertex_colourx,'FaceAlpha','interp',...
    'EdgeColor','none','CDataMapping','direct', ...
    'AlphaDataMapping','direct','FaceVertexAlphaData', vertex_alphax);
axis equal
axis off
